//Modified part of XDBEngine source code.
//Version 1.5
//Created by Tomas Koutny.
//For more info see http://www.rawos.com/en

//definitions of internal relations
unit AbstractLayer;

interface

uses IStream64Cache, Stream64;

const
   PageSize = 4*1024;   //maximum size of memory allocated at one TCachePage
   PageSHR = 12;        //2^PageSHR = PageSize
   BTreeN = 8;          //rank of used BTree


type
   PCachePage = ^TCachePage;
   TCachePage = record
                  Data:pointer;         //points to allocated memory that
                                        //stores cached data
                  Length:LongWord;      //really loaded count of bytes
                                        //i.e. when cached end of file
                                        //and FileSize mod PageSize > 0
                  Loaded:boolean;       //true if data has been already loaded
                                        //in this page
                                        //if false then write to Data must
                                        //be done first
                end;


   TAEEngine = class(TACache64Engine)
               public
                 //TAEEngine has to provide some mechanism how read & write
                 //through cache
                 function RequestPage(AStream:TStream64; APosition:Int64):
                                                  PCachePage; virtual; abstract;
                      //this function is to be called by TAECacheStream
                      //returns page that holds positions from APosition*PageSize
                      //to APosition*(PageSize+1)-1
                      //nil can not be returned
                 function FindPage(AStream:TStream64; APosition:Int64):
                                                  PCachePage; virtual; abstract;
                      //same meaning as RequestPage but can return nil if
                      //page is not cached => useful for call as TCachedStream.
                      //SetSize that doesn't need to load data into the cache
                 procedure InvalidatePages(AStream:TStream64);virtual; abstract;
                      //invalidates contents of all pages that belong
                      //to the stream AStream
               end;

   TAECacheStream = class(TStream64)
                    public
                      constructor Create(AOwner:TAEEngine; AStream:TStream64);
                                                              virtual; abstract;
                    end;    

implementation

end.
