//XDBEngine cache test program / Version modified for Stream64Cache
//Version 1.0; Revision 1
//Created by Tomas Koutny.
//for more info see http://www.rawos.com/xdbe/index.html
//This file is public domain.


//This program demonstrates how to use cache if you're gonna develop your own
//drivers or to use it in another way.
//Program knows 2 kind of tests. CTest tests cache on small data blocks and
//verifies that they has been written correctly. QTest performs test with
//larger data blocks without any additional verifications, but tests more times.


//To fully understand how to use cache or only to explain test result
//see related www page.

program XCache;

uses
  Forms,
  SysUtils,
  Windows,
  stream64 in '..\..\common\stream64.pas',
  IStream64Cache in '..\interface\IStream64Cache.pas';

{$R *.RES}

const
   FSize = 2*1024*1024; //determines the size of file to be used for test
   FCacheSize = 4*1024*1024;


//we have to import only one function
function CreateCacheEngine(CacheSize:integer):TACache64Engine;
                                                   external 'Stream64Cache.dll';


var stream:TStream64;
    FName:string;
    cache:TACache64Engine;

function CTest(AStream:TStream64):string;
var buf:packed record
                 num1:byte;
                 num2:longword;
                 num3:word;
               end;
    bsize:integer;
    i, tested:dword;
    ticks:dword;


begin
  bsize:=sizeof(buf);
  result:='';

  tested:=0;
  ticks:=GetTickCount;
  i:=0;
  while tested<FSize do
    begin
      with buf do
        begin
          num1:=i;
          num2:=i;
          num3:=i;
        end;
      AStream.Write(buf, bsize);
      inc(tested, bsize);
    end;
  ticks:=GetTickCount - ticks;
  result:='   Buffer write: '+inttostr(ticks)+#10;

  AStream.Position:=0;
  tested:=0;
  ticks:=GetTickCount;
  i:=0;
  while tested<FSize do
    begin
      AStream.Read(buf, bsize);
      with buf do
        if (num1<>i) or (num2<>i) or (num3<>i) then
         begin
           Application.MessageBox(pchar('Stream '+'"'+AStream.ClassName+'"'+
                                  ' failed to read proper data!'#10+
                                  'expected: '+inttostr(i)+#10+
                                  'read(1/2/3: '+inttostr(num1)+
                                  '/'+inttostr(num2)+
                                  '/'+inttostr(num3)),
                                  'Cache test error',
                                  MB_OK or MB_ICONSTOP);
           AStream.Free;
           Halt;
         end;

      inc(tested, bsize);
    end;
  ticks:=GetTickCount - ticks;
  result:=result+'   Buffer read: '+inttostr(ticks)+#10;
end;

function QTest(AStream:TSTream64):string;
var buf:packed array[1..128*1024] of byte;
    tested:integer;
    bsize:cardinal;
    ticks:dword;

function testwrite:string;
begin
  tested:=0;
  ticks:=GetTickCount;
  AStream.Position:=0;
  while tested<FSize do
    begin
      AStream.Write(buf, bsize);
      inc(tested, bsize);
    end;
  ticks:=GetTickCount - ticks;
  result:=inttostr(ticks)+#10;
end;

function testread:string;
begin
  AStream.Position:=0;
  tested:=0;
  ticks:=GetTickCount;
  while tested<FSize do
    begin
      AStream.Read(buf, bsize);
      inc(tested, bsize);
    end;
  ticks:=GetTickCount - ticks;
  result:=inttostr(ticks)+#10;
end;

begin
  bsize:=sizeof(buf);
  result:='   1st Buffer write:  '+testwrite;
  result:=result+'   1st buffer read:  '+testread;
  result:=result+'   2nd buffer read: '+testread;
  result:=result+'   2nd Buffer write: '+testwrite;
end;

var fmsg:string;

begin
  Application.Initialize;

  //Get file name
  FName:=extractfilepath(paramstr(0));
  if FName[length(FName)]<>'\' then FName:=FName+'\';
  FName:=FName+'test.data';

  //create non cached stream
  Stream:=TFileStream64.Create(FName, fmCreate);
//  fmsg:='Without cache:'#10+CTest(Stream);
  fmsg:='Without cache:'#10+QTest(Stream);
  Stream.Free;

  //try to read it with taken of cache
  Stream:=TFileStream64.Create(FName, fmCreate);
  Cache:=TACache64Engine(CreateCacheEngine(FCacheSize));
  Stream:=cache.CreateCacheStream(Stream);
//  fmsg:=fmsg+'With cache:'#10+CTest(Stream);
  fmsg:=fmsg+'With cache:'#10+QTest(Stream);
  Stream.Free;

  Application.MessageBox(pchar(fmsg),
                         'Cache test results',
                         MB_OK or MB_ICONINFORMATION);

  Cache.Free;
end.
